@@ -1,9 +1,12 @@
 #define _LARGEFILE64_SOURCE     /* required for GLIBC to enable stat64 and friends */
+#include "doassert.h"
 #include <sys/types.h>
 #include <regex.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "error.h"
@@ -12,9 +15,19 @@
 #include "exec.h"
 #include "utils.h"
 #include "globals.h"
+#include "ui.h"
+
+color_offset_in_line *realloc_color_offset_in_line(color_offset_in_line *oldp, int n_entries, char *file, const char *function, int line)
+{
+	assert(n_entries > 0);
+
+	return (color_offset_in_line *)myrealloc(oldp, n_entries * sizeof(color_offset_in_line), file, function, line);
+}
 
 void add_color_scheme(int_array_t *schemes, int cur_scheme)
 {
+	assert(cur_scheme >= 0);
+
 	add_to_iat(schemes, cur_scheme);
 }
 
@@ -23,6 +36,8 @@ int gen_color(char *start, char *end)
 	char *loop;
 	int chk = 0;
 
+	assert(end != NULL);
+
 	for(loop=start; loop<end; loop++)
 	{
 		chk ^= *loop;
@@ -42,14 +57,15 @@ myattr_t gen_syslog_progname_color(char *string)
 			string = strchr(&string[16], ' ');
 		}
 
-		while(string && isspace(*string)) string++;
-
 		if (string)
 		{
-			char *end1 = strchr(string, '[');
-			char *end2 = strchr(string, ' ');
-			char *end3 = strchr(string, ':');
-			char *end = NULL;
+			char *end1, *end2, *end3, *end = NULL;
+
+			while(isspace(*string)) string++;
+
+			end1 = strchr(string, '[');
+			end2 = strchr(string, ' ');
+			end3 = strchr(string, ':');
 
 			end = end1;
 			if ((end2 && end2 < end) || (end == NULL))
@@ -71,28 +87,20 @@ myattr_t gen_color_from_field(char *string, char *field_del, int field_nr)
 {
 	myattr_t cdev = { -1, -1 };
 
+	assert(field_nr >= 0);
+
 	if (use_colors)
 	{
-		int nlen = strlen(field_del), loop;
+		int field_del_len = strlen(field_del), loop;
 		char *dummy = NULL;
 
 		for(loop=0; loop<field_nr; loop++)
 		{
 			string = strstr(string, field_del);
-			while(string)
-			{
-				if (strncmp(string, field_del, nlen) == 0)
-				{
-					string += nlen;
-				}
-				else
-				{
-					break;
-				}
-			}
-
 			if (!string)
 				break;
+
+			string += field_del_len;
 		}
 
 		if (string)
@@ -113,8 +121,8 @@ void color_script(char *line, script *pscript, int *cur_n_cmatches, color_offset
 {
 	char finished = 0;
 	int line_len = min(SCRIPT_IO_BUFFER_SIZE - 2, strlen(line));
-	char *iobuffer = (char *)mymalloc(SCRIPT_IO_BUFFER_SIZE, "script i/o buffer");
-	char *line_buf = (char *)mymalloc(line_len + 1 + 1, "color_script");
+	char *iobuffer = (char *)mymalloc(SCRIPT_IO_BUFFER_SIZE, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+	char *line_buf = (char *)mymalloc(line_len + 1 + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 	exec_script(pscript);
 
@@ -151,11 +159,15 @@ void color_script(char *line, script *pscript, int *cur_n_cmatches, color_offset
 			if (*cur_n_cmatches == *n_cmatches)
 			{
 				*cur_n_cmatches = (*cur_n_cmatches + 8) * 2;
-				*cmatches = (color_offset_in_line *)myrealloc(*cmatches, *cur_n_cmatches * sizeof(color_offset_in_line), "color_script: cmatches");
+				*cmatches = realloc_color_offset_in_line(*cmatches, *cur_n_cmatches, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			}
 
 			komma = strchr(workpnt, ',');
-			if (!komma) error_exit("malformed line returned by '%s'\n", pscript -> script);
+			if (!komma)
+			{
+				error_popup("Color scheme script", -1, "Malformed line returned by color selection script '%s': first ',' missing.\n", pscript -> script);
+				break;
+			}
 
 			memset(&(*cmatches)[*n_cmatches], 0x00, sizeof(color_offset_in_line));
 
@@ -163,7 +175,11 @@ void color_script(char *line, script *pscript, int *cur_n_cmatches, color_offset
 			(*cmatches)[*n_cmatches].end   = atoi(komma + 1);
 
 			komma = strchr(komma + 1, ',');
-			if (!komma) error_exit("malformed line returned by '%s'\n", pscript -> script);
+			if (!komma)
+			{
+				error_popup("Color scheme script", -1, "Malformed line returned by color selection script '%s': second ',' missing.\n", pscript -> script);
+				break;
+			}
 
 			(*cmatches)[*n_cmatches].attrs = parse_attributes(komma + 1);
 			(*n_cmatches)++;
@@ -235,7 +251,7 @@ void get_colors_from_colorscheme(char *line, int_array_t *color_schemes, color_o
 					if (flags == CSREFLAG_CMP_VAL_LESS || flags == CSREFLAG_CMP_VAL_BIGGER || flags == CSREFLAG_CMP_VAL_EQUAL)
 					{
 						int val_size = this_end_offset - this_start_offset;
-						char *valstr = (char *)mymalloc(val_size + 1, "get_colors_from_colorscheme: compare value");
+						char *valstr = (char *)mymalloc(val_size + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 						double value = 0.0;
 						char value_match = 0;
 
@@ -247,9 +263,9 @@ void get_colors_from_colorscheme(char *line, int_array_t *color_schemes, color_o
 
 						if (flags == CSREFLAG_CMP_VAL_LESS && value < cschemes[get_iat_element(color_schemes, loop)].pentries[cs_index].cmp_value)
 							value_match = 1;
-						if (flags == CSREFLAG_CMP_VAL_BIGGER && value > cschemes[get_iat_element(color_schemes, loop)].pentries[cs_index].cmp_value)
+						else if (flags == CSREFLAG_CMP_VAL_BIGGER && value > cschemes[get_iat_element(color_schemes, loop)].pentries[cs_index].cmp_value)
 							value_match = 1;
-						if (flags == CSREFLAG_CMP_VAL_EQUAL && value == cschemes[get_iat_element(color_schemes, loop)].pentries[cs_index].cmp_value)
+						else if (flags == CSREFLAG_CMP_VAL_EQUAL && value == cschemes[get_iat_element(color_schemes, loop)].pentries[cs_index].cmp_value)
 							value_match = 1;
 
 						if (!value_match)
@@ -259,7 +275,7 @@ void get_colors_from_colorscheme(char *line, int_array_t *color_schemes, color_o
 					if (cur_n_cmatches == *n_cmatches)
 					{
 						cur_n_cmatches = (cur_n_cmatches + 8) * 2;
-						*cmatches = (color_offset_in_line *)myrealloc(*cmatches, cur_n_cmatches * sizeof(color_offset_in_line), "get_colors_from_colorscheme: cmatches");
+						*cmatches = realloc_color_offset_in_line(*cmatches, cur_n_cmatches, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 					}
 
 					memset(&(*cmatches)[*n_cmatches], 0x00, sizeof(color_offset_in_line));
@@ -299,7 +315,6 @@ myattr_t choose_color(char *string, proginfo *cur, color_offset_in_line **cmatch
 
 	if (cur -> cdef.term_emul != TERM_IGNORE)
 	{
-		LOG("terminal: %d\n", cur -> cdef.term_emul);
 		*new_string = emulate_terminal(string, cmatches, n_cmatches);
 		return find_attr(COLOR_WHITE, -1, -1);
 	}
@@ -344,6 +359,10 @@ myattr_t choose_color(char *string, proginfo *cur, color_offset_in_line **cmatch
 			/* colorscheme or colorscripts */
 			get_colors_from_colorscheme(string, &cur -> cdef.color_schemes, cmatches, n_cmatches, has_merge_colors);
 			break;
+
+		default:
+			assert(0);
+			break;
 	}
 
 	if (cdev.colorpair_index == -1)
@@ -376,8 +395,8 @@ void init_colors(void)
 	cp.size = min(256, COLOR_PAIRS);
 	if (cp.size)
 	{
-		cp.fg_color = (int *)mymalloc(cp.size * sizeof(int), "main: fg colors");
-		cp.bg_color = (int *)mymalloc(cp.size * sizeof(int), "main: bg colors");
+		cp.fg_color = (int *)mymalloc(cp.size * sizeof(int), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+		cp.bg_color = (int *)mymalloc(cp.size * sizeof(int), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	}
 
 	(void)find_or_init_colorpair(-1, -1, 1);		/* colorpair 0 is always default, cannot be changed */
@@ -391,5 +410,5 @@ void init_colors(void)
 	/*	(void)find_or_init_colorpair(COLOR_BLACK, -1, 1); */
 
 	if (use_colors && cp.n_def != DEFAULT_COLORPAIRS)
-		error_exit("unexpected number of colors: %d (%d)\n", cp.n_def, DEFAULT_COLORPAIRS);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Unexpected number of colors: %d (%d)\n", cp.n_def, DEFAULT_COLORPAIRS);
 }
