@@ -9,6 +9,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "error.h"
@@ -17,6 +19,7 @@
 #include "term.h"
 #include "my_pty.h"
 #include "globals.h"
+#include "ui.h"
 
 
 int start_tail(char *filename, char retry_open, char follow_filename, int initial_tail, int *pipefd)
@@ -94,7 +97,7 @@ int start_tail(char *filename, char retry_open, char follow_filename, int initia
 #endif
 		{
 			/* check the posix compliance level */
-			if (posix_version && atoi(posix_version) >= 200112)
+			if ((posix_version && atoi(posix_version) >= 200112) || posix_tail == MY_TRUE)
 			{
 				pars[npars++] = "-f";
 				pars[npars++] = "-n";
@@ -114,10 +117,10 @@ int start_tail(char *filename, char retry_open, char follow_filename, int initia
 		pars[npars] = NULL;
 
 		/* run tail! */
-		if (-1 == execvp(pars[0], pars)) error_exit("start_tail: execvp of tail failed\n");
+		if (-1 == execvp(pars[0], pars)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while starting process %s.\n", pars[0]);
 
 		/* if execlp returns, an error occured */
-		error_exit("start_tail: error while starting process!\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "An error occured while starting process %s!\n", pars[0]);
 	}
 
 	return pid;
@@ -125,7 +128,7 @@ int start_tail(char *filename, char retry_open, char follow_filename, int initia
 
 void start_proc_signal_handler(int sig)
 {
-	if (sig != SIGTERM) error_exit("unexpected signal %d in start_proc_signalhandler\n", sig);
+	if (sig != SIGTERM) error_popup("Signal handler(2)", -1, "Unexpected signal %d.\n", sig);
 
 	stop_process(tail_proc);
 
@@ -136,13 +139,13 @@ int start_proc(proginfo *cur, int initial_tail)
 {
 	cur -> n_runs++;
 
-	if (cur -> is_command)
+	if (cur -> wt == WT_COMMAND)
 	{
 		int fd_master, fd_slave;
 
 		/* allocate pseudo-tty & fork*/
 		cur -> pid = get_pty_and_fork(&fd_master, &fd_slave);
-		if (-1 == cur -> pid) error_exit("start_proc: failed to create pseudo-tty & fork\n");
+		if (-1 == cur -> pid) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "An error occured while invoking get_pty_and_fork.\n");
 
 		/* child? */
 		if (cur -> pid == 0)
@@ -160,10 +163,10 @@ int start_proc(proginfo *cur, int initial_tail)
 			setup_for_childproc(fd_slave, 1, term_t_to_string(cur -> cdef.term_emul));
 
 			/* start process */
-			if (-1 == execlp(shell, shell, "-c", cur -> filename, (void *)NULL)) error_exit("start_proc: execlp of %s failed\n", cur -> filename);
+			if (-1 == execlp(shell, shell, "-c", cur -> filename, (void *)NULL)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while starting \"%s -c '%s'\".\n", shell, cur -> filename);
 
 			/* if execlp returns, an error occured */
-			error_exit("start_proc: error while starting process!\n");
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while starting process %s!\n", shell);
 		}
 
 #if defined(sun) || defined(__sun) || defined(AIX) || defined(_HPUX_SOURCE) || defined(OSF1) || defined(scoos)
@@ -172,7 +175,7 @@ int start_proc(proginfo *cur, int initial_tail)
 		 *
 		 */
 #else
-		if (myclose(fd_slave) == -1) error_exit("start_proc: error closing slave-fd (pseudo tty, fd %d)\n", fd_slave);
+		if (myclose(fd_slave) == -1) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "An error occured while closing the slave fd (pseudo tty, fd %d)\n", fd_slave);
 #endif
 
 		/* remember master-fd (we'll read from that one) */
@@ -182,12 +185,12 @@ int start_proc(proginfo *cur, int initial_tail)
 		/* next time, sleep */
 		cur -> restart.first = 0;
 	}
-	else
+	else if (cur -> wt == WT_FILE)
 	{
 		int pipefd[2];
 
 		/* create a pipe, will be to child-process */
-		if (-1 == pipe(pipefd)) error_exit("start_proc: error creating pipe\n");
+		if (-1 == pipe(pipefd)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error creating pipe.\n");
 
 		if (cur -> check_interval)
 		{
@@ -218,7 +221,6 @@ int start_proc(proginfo *cur, int initial_tail)
 						 */
 						if (cur_file && strcmp(new_file, cur_file) != 0)
 						{
-							LOG("killing process for %s\n", cur_file);
 							stop_process(tail_proc);
 						}
 
@@ -226,25 +228,22 @@ int start_proc(proginfo *cur, int initial_tail)
 						myfree(cur_file);
 						cur_file = new_file;
 
-						LOG("new_file: %s\n", cur_file);
-
 						/* and start a proc process */
 						tail_proc = start_tail(cur_file, cur -> retry_open, cur -> follow_filename, initial_tail, pipefd);
 						if (tail_proc == -1)
 						{
-							LOG("failed starting process for file %s: %d\n", cur_file, errno);
 							break;
 						}
 					}
 					else
 					{
-						LOG("no file found for pattern %s\n", cur -> filename);
+						/* LOG("no file found for pattern %s\n", cur -> filename); */
 					}
 
 					sleep(cur -> check_interval);
 				}
 
-				LOG("stopped checking for file pattern %s\n", cur -> filename);
+				/* LOG("stopped checking for file pattern %s\n", cur -> filename); */
 
 				exit(1);
 			}
@@ -289,20 +288,20 @@ int execute_program(char *execute, char bg)
 			setup_for_childproc(open_null(), 1, "dumb");
 
 		/* start process */
-		if (-1 == execlp(shell, shell, "-c", execute, (void *)NULL)) error_exit("execute_program: execlp of %s failed\n", execute);
+		if (-1 == execlp(shell, shell, "-c", execute, (void *)NULL)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while starting \"%s -c '%s'\".\n", execute);
 
 		/* if execlp returns, an error occured */
-		error_exit("execute_program: error while starting process!\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while starting process!\n");
 	}
 	else if (child == -1)
 	{
-		error_exit("execute_program: failed to fork child process\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Failed to fork child process.\n");
 	}
 
 	if (bg)
 	{
 		/* remember this childprocess: we'll see if it has
-		 * exited in the main-loop
+		 * died in the main-loop
 		 */
 		children_list[n_children++] = child;
 	}
@@ -310,7 +309,7 @@ int execute_program(char *execute, char bg)
 	{
 		/* wait for the childprocess to exit */
 		if (waitpid(child, &status, 0) == -1)
-			error_exit("execute_program: failt waiting for process to exit!\n");
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while waiting for process to exit.\n");
 
 		/* restore (n)curses */
 		mydoupdate();
@@ -330,31 +329,31 @@ pid_t exec_with_pipe(char *command, int pipe_to_proc[], int pipe_from_proc[])
 	pid_t pid = -1;
 
 	if (pipe(pipe_to_proc) == -1)
-		error_exit("exec_with_pipe: error creating pipe 1\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error creating pipe.\n");
 
 	if (pipe(pipe_from_proc) == -1)
-		error_exit("exec_with_pipe: error creating pipe 2\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error creating pipe.\n");
 
 	if ((pid = fork()) == -1)
-		error_exit("exec_with_pipemain: fork() failed\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "fork() failed.\n");
 
 	if (pid == 0)
 	{
-		close(0);
-		if (mydup(pipe_to_proc[0]) == -1) error_exit("exec_with_pipe: dup() failed\n");
-		close(pipe_to_proc[1]); /* will not write to itself, only parent writes to it */
-		close(1);
-		close(2);
-		if (mydup(pipe_from_proc[1]) == -1) error_exit("exec_with_pipe: dup() failed\n");
-		if (mydup(pipe_from_proc[1]) == -1) error_exit("exec_with_pipe: dup() failed\n");
-		close(pipe_from_proc[0]);
+		myclose(0);
+		if (mydup(pipe_to_proc[0]) == -1)   error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "dup() failed.\n");
+		myclose(pipe_to_proc[1]); /* will not write to itself, only parent writes to it */
+		myclose(1);
+		myclose(2);
+		if (mydup(pipe_from_proc[1]) == -1) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "dup() failed.\n");
+		if (mydup(pipe_from_proc[1]) == -1) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "dup() failed.\n");
+		myclose(pipe_from_proc[0]);
 
 		/* start process */
-/*		if (-1 == execlp(shell, shell, "-c", command, (void *)NULL)) error_exit("main: execlp of %s failed\n", command); */
-		if (-1 == execlp(command, command, (void *)NULL)) error_exit("bla\n");
+/*		if (-1 == execlp(shell, shell, "-c", command, (void *)NULL)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "execlp of %s failed\n", command); */
+		if (-1 == execlp(command, command, (void *)NULL)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while starting '%s'.\n", command);
 
 		/* if execlp returns, an error occured */
-		error_exit("exec_with_pipe: error while starting process '%s'!\n", command);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while starting process '%s'.\n", command);
 	}
 
 	return pid;
@@ -364,19 +363,19 @@ pid_t exec_with_pty(char *command, int *fd)
 {
 	int fd_master = -1, fd_slave = -1;
 	pid_t pid = get_pty_and_fork(&fd_master, &fd_slave);
-	if (-1 == pid) error_exit("start_proc: failed to create pseudo-tty & fork\n");
+	if (-1 == pid) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "An error occured while invoking get_pty_and_fork.\n");
 
 	if (pid == 0)
 	{
 		setpgid(0,0);
 
-		close(fd_master);
+		myclose(fd_master);
 
 		/* connect slave-fd to stdin/out/err */
 		setup_for_childproc(fd_slave, 1, "dumb");
 
 		/* start process */
-		if (-1 == execlp(command, command, (void *)NULL)) error_exit("exec_with_pty: execlp of %s failed\n", command);
+		if (-1 == execlp(command, command, (void *)NULL)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while starting '%s'.\n", command);
 	}
 
 	*fd = fd_master;
@@ -384,7 +383,7 @@ pid_t exec_with_pty(char *command, int *fd)
 #if defined(sun) || defined(__sun) || defined(AIX) || defined(_HPUX_SOURCE) || defined(OSF1) || defined(scoos)
 	/* see start_proc */
 #else
-	if (myclose(fd_slave) == -1) error_exit("exec_with_pty: error closing slave-fd (pseudo tty, fd %d)\n", fd_slave);
+	if (myclose(fd_slave) == -1) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error closing slave-fd (pseudo tty, fd %d)\n", fd_slave);
 #endif
 
 	return pid;
