@@ -3,8 +3,11 @@
 #include <regex.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
+#include "mem.h"
 
 NEWWIN **splitlines = NULL;
 NEWWIN *menu_win = NULL;
@@ -13,12 +16,15 @@ char *config_file = NULL;
 char *mail_spool_file = NULL;
 char *nsubwindows = NULL;
 char *set_title = NULL;
-char *shell = "/bin/sh";
-char *tail = "tail";
-char *ts_format = "%b %d %H:%M:%S";
-char *statusline_ts_format = "%b %d %H:%M:%S %Y";
-char *cnv_ts_format = "%b %d %H:%M:%S %Y";
-char *line_ts_format = "%b %d %H:%M:%S %Y ";
+char *shell = NULL;
+char *tail = NULL;
+char *ts_format = NULL;
+char *statusline_ts_format = NULL;
+char *syslog_ts_format = NULL;
+char *cnv_ts_format = NULL;
+char *line_ts_format = NULL;
+char *window_number = NULL;
+char *subwindow_number = NULL;
 char **color_names = NULL;
 char *replace_by_markerline = NULL;
 const char *F1 = "For help at any time press F1.";
@@ -36,6 +42,7 @@ editscheme *pes = NULL;
 int n_es = 0;
 char *defaultcscheme = NULL;
 proginfo *terminal_index = NULL;
+char *global_find = NULL;
 
 int terminal_main_index = -1;
 int default_color_scheme = -1;
@@ -63,13 +70,16 @@ int did_n_beeps = 0;
 int default_min_shrink = 10;
 int wordwrapmaxlength = 10;
 int default_bg_color = -1;
+int total_wakeups = 0;
+int split = 0;
+int syslog_port = 514;
 
 double heartbeat_interval = 0.0;
 double heartbeat_t = 0.0;
 off64_t msf_prev_size = 0;
 
-time_t msf_last_check = 0;
-time_t mt_started;
+dtime_t msf_last_check = 0;
+dtime_t mt_started;
 
 pid_t children_list[MAX_N_SPAWNED_PROCESSES];
 pid_t tail_proc = 0;	/* process used by checker-proc */
@@ -109,7 +119,6 @@ char mail = 0;
 char no_linewrap = 0;
 char prev_term_char = -1;
 char show_subwindow_id = 0;
-char split = 0;
 char tab_width = 4;		/* some people use 8 */
 char terminal_changed = 0;
 char timestamp_in_markerline = 0;
@@ -133,9 +142,20 @@ char abort_key = 7;
 char exit_key = 3;
 char default_sb_showwinnr = 0;
 char reuse_searchstring = 1;
+char need_died_procs_check = 0;
+char scrollback_no_colors = 0;
+char scrollback_search_new_window = 1;
+mybool_t posix_tail = 0;
+mybool_t resolv_ip_addresses = 1;
+mybool_t show_severity_facility = 1;
 
 regex_t global_highlight_re;
 char *global_highlight_str = NULL;
 
 filterscheme *pfs = NULL;
 int n_fs = 0;
+
+char *sigs[] = { NULL, "SIGHUP", "SIGINT", "SIGQUIT", "SIGILL", "SIGTRAP", "SIGABRT/SIGIOT", "SIGIOT", "SIGBUS", "SIGFPE", "SIGKILL", "SIGUSR1", "SIGSEGV", "SIGUSR2", "SIGPIPE", "SIGALRM", "SIGTERM", "SIGSTKFLT", "SIGCHLD", "SIGCONT", "SIGSTOP", "SIGTSTP", "SIGTTIN", "SIGTTOU", "SIGURG", "SIGXCPU", "SIGXFSZ", "SIGVTALRM", "SIGPROF", "SIGWINCH", "SIGIO/SIGPOLL", "SIGPWR", "SIGSYS/SIGUNUSED" };
+int n_known_sigs = 32;
+char *severities[8] = { "emerg", "alert", "crit", "err", "warning", "notice", "info", "debug" };
+char *facilities[24] = { "kern", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", "authpriv", "ftp", "?", "?", "?", "?", "local0", "local1", "local2", "local3", "local4", "local5", "local6", "local7" };
