@@ -3,7 +3,7 @@ extern buffer *lb;
 extern int nfd;
 extern int max_y, max_x;
 extern char terminal_changed;
-extern char split;
+extern int split;
 extern char banner;
 extern NEWWIN **splitlines;
 extern int n_splitlines;
@@ -34,7 +34,7 @@ extern char bright_colors;
 extern char *mail_spool_file;
 extern struct stat64 msf_info;
 extern off64_t msf_prev_size;
-extern time_t msf_last_check;
+extern dtime_t msf_last_check;
 extern int update_interval;
 extern char *tail;
 extern char afs;
@@ -118,6 +118,9 @@ extern history_t search_h;
 extern history_t cmdfile_h;
 extern int default_bg_color;
 extern char reuse_searchstring;
+extern char *sigs[];
+extern int n_known_sigs;
+extern char need_died_procs_check;
 extern chtype box_bottom_left_hand_corner;
 extern chtype box_bottom_right_hand_corner;
 extern chtype box_bottom_side;
@@ -126,3 +129,16 @@ extern chtype box_right_side;
 extern chtype box_top_left_hand_corner;
 extern chtype box_top_right_hand_corner;
 extern chtype box_top_side;
+extern char *window_number;
+extern char *subwindow_number;
+extern int total_wakeups;
+extern mybool_t posix_tail;
+extern mybool_t resolv_ip_addresses;
+extern mybool_t show_severity_facility;
+extern char *severities[];
+extern char *facilities[];
+extern char *syslog_ts_format;
+extern char scrollback_no_colors;
+extern int syslog_port;
+extern char scrollback_search_new_window;
+extern char *global_find;
