@@ -6,6 +6,8 @@
 #include <stdlib.h>
 #include <pwd.h>
 #include <errno.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "error.h"
@@ -18,14 +20,15 @@
 #include "misc.h"
 #include "globals.h"
 
+int scrollback_search_to_new_window(buffer *pbuf, char *org_title, char *find_str, mybool_t case_insensitive);
 
-int find_string(buffer *pbuf, char *find, int offset, char direction)
+int find_string(buffer *pbuf, char *find, int offset, char direction, mybool_t case_insensitive)
 {
 	int loop, index = -1, rc;
 	regex_t regex;
 
 	/* compile the searchstring (which can be a regular expression) */
-	if ((rc = regcomp(&regex, find, REG_EXTENDED | re_case_insensitive?REG_ICASE:0)))
+	if ((rc = regcomp(&regex, find, REG_EXTENDED | (case_insensitive == MY_TRUE?REG_ICASE:0))))
 	{
 		regexp_error_popup(rc, &regex);
 
@@ -36,6 +39,9 @@ int find_string(buffer *pbuf, char *find, int offset, char direction)
 	{
 		for(loop=offset; loop<pbuf -> curpos; loop++)
 		{
+			if ((pbuf -> be)[loop].Bline == NULL)
+				continue;
+
 			if (regexec(&regex, (pbuf -> be)[loop].Bline, 0, NULL, 0) == 0)
 			{
 				index = loop;
@@ -43,10 +49,13 @@ int find_string(buffer *pbuf, char *find, int offset, char direction)
 			}
 		}
 	}
-	else if (direction == -1)
+	else if (direction == (char)-1)
 	{
 		for(loop=offset; loop>=0; loop--)
 		{
+			if ((pbuf -> be)[loop].Bline == NULL)
+				continue;
+
 			if (regexec(&regex, (pbuf -> be)[loop].Bline, 0, NULL, 0) == 0)
 			{
 				index = loop;
@@ -60,14 +69,14 @@ int find_string(buffer *pbuf, char *find, int offset, char direction)
 	return index;
 }
 
-void scrollback_find_popup(char **find_str)
+void scrollback_find_popup(char **find_str, mybool_t *pcase_insensitive)
 {
 	char *dummy;
 	NEWWIN *mywin = create_popup(5, 40);
 
 	win_header(mywin, "Find");
 
-	dummy = edit_string(mywin, 3, 2, 40, 80, 0, *find_str, HELP_SCROLLBACK_EDIT_SEARCH_STRING, -1);
+	dummy = edit_string(mywin, 3, 2, 40, 80, 0, reuse_searchstring?*find_str:NULL, HELP_SCROLLBACK_EDIT_SEARCH_STRING, -1, &search_h, pcase_insensitive);
 	myfree(*find_str);
 	*find_str = dummy;
 
@@ -82,7 +91,7 @@ void scrollback_savefile(buffer *pbuf)
 	win_header(mywin, "Save buffer to file");
 
 	mvwprintw(mywin -> win, 4, 2, "Select file");
-	file = edit_string(mywin, 5, 2, 40, find_path_max(), 0, NULL, HELP_SCROLLBACK_SAVEFILE_ENTER_FILENAME, -1);
+	file = edit_string(mywin, 5, 2, 40, find_path_max(), 0, NULL, HELP_SCROLLBACK_SAVEFILE_ENTER_FILENAME, -1, &cmdfile_h, NULL);
 	if (file)
 	{
 		FILE *fh = fopen(file, "w");
@@ -108,7 +117,7 @@ void scrollback_savefile(buffer *pbuf)
 					}
 					if (display)
 					{
-						fprintf(fh, "%s\n", (pbuf -> be)[loop].Bline?(pbuf -> be)[loop].Bline:"");
+						fprintf(fh, "%s\n", USE_IF_SET((pbuf -> be)[loop].Bline, ""));
 					}
 
 					if (pmatch) myfree(pmatch);
@@ -134,29 +143,37 @@ int get_lines_needed(char *string, int terminal_width)
 		return 1;
 }
 
-void scrollback_displayline(NEWWIN *win, buffer *pbuf, int buffer_offset, int terminal_offset, int offset_in_line, mybool_t force_to_winwidth)
+void scrollback_displayline(int window_nr, NEWWIN *win, buffer *pbuf, int buffer_offset, int terminal_offset, int offset_in_line, mybool_t force_to_winwidth, char show_winnr)
 {
-	proginfo *cur_line_meta = (pbuf -> be)[buffer_offset].pi;
 	char *cur_line = (pbuf -> be)[buffer_offset].Bline;
-	regmatch_t *pmatch = NULL;
-	int matching_regex = -1;
-	char *error = NULL;
 
 	wmove(win -> win, terminal_offset, 0);
 
 	if (cur_line)
 	{
+		proginfo *cur_line_meta = (pbuf -> be)[buffer_offset].pi;
+		double ts = (pbuf -> be)[buffer_offset].ts;
+		char old_color_settings = 0;
+
+		if (scrollback_no_colors && cur_line_meta != NULL)
+		{
+			old_color_settings = cur_line_meta -> cdef.colorize;
+			cur_line_meta -> cdef.colorize = 0;
+		}
 		if (IS_MARKERLINE(cur_line_meta))
 		{
-			color_print(win, cur_line_meta, cur_line, NULL, -1, MY_FALSE, 0, 0);
+			color_print(window_nr, win, cur_line_meta, cur_line, NULL, -1, MY_FALSE, 0, 0, ts, show_winnr);
 		}
 		else /* just a buffered line */
 		{
+			char *error = NULL;
+			regmatch_t *pmatch = NULL;
+			int matching_regex = -1;
 			char display;
 			(void)check_filter(cur_line_meta, cur_line, &pmatch, &error, &matching_regex, 0, &display);
 			if (error)
 			{
-				color_print(win, cur_line_meta, error, NULL, -1, MY_FALSE, 0, 0);
+				color_print(window_nr, win, cur_line_meta, error, NULL, -1, MY_FALSE, 0, 0, ts, show_winnr);
 				myfree(error);
 			}
 			if (display)
@@ -169,35 +186,42 @@ void scrollback_displayline(NEWWIN *win, buffer *pbuf, int buffer_offset, int te
 					if (offset_in_line < line_len)
 						new_size = min(win -> width, line_len - offset_in_line);
 
-					color_print(win, cur_line_meta, cur_line, pmatch, matching_regex, force_to_winwidth, offset_in_line, offset_in_line + new_size);
+					color_print(window_nr, win, cur_line_meta, cur_line, pmatch, matching_regex, force_to_winwidth, offset_in_line, offset_in_line + new_size, ts, show_winnr);
 				}
 				else
 				{
-					color_print(win, cur_line_meta, cur_line, pmatch, matching_regex, force_to_winwidth, 0, 0);
+					color_print(window_nr, win, cur_line_meta, cur_line, pmatch, matching_regex, force_to_winwidth, 0, 0, ts, show_winnr);
 				}
 			}
+
+			myfree(pmatch);
+		}
+		if (scrollback_no_colors && cur_line_meta != NULL)
+		{
+			cur_line_meta -> cdef.colorize = old_color_settings;
 		}
 	}
 	else /* an empty line */
 	{
 		/* do nothing */
 	}
-
-	myfree(pmatch);
 }
 
-void scrollback_do(buffer *pbuf, char *header)
+int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 {
+	int rc = 0;
 	char *find = NULL;
 	NEWWIN *mywin1, *mywin2;
 	int nlines = max_y - 6, ncols = max_x - 6;
-	int offset = max(0, pbuf -> curpos - nlines);
+	int offset = max(0, pbuf -> curpos - nlines); // FIXME: aantal regels laten afhangen van lengte
 	char redraw = 2;
 	int line_offset = 0;
+	char show_winnr = default_sb_showwinnr;
+	mybool_t case_insensitive = re_case_insensitive;
 
 	if (global_highlight_str)
 	{
-		find = mystrdup(global_highlight_str, "scrollback_do: global highlight str");
+		find = mystrdup(global_highlight_str, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	}
 
 	mywin1 = create_popup(max_y - 4, max_x - 4);
@@ -214,28 +238,56 @@ void scrollback_do(buffer *pbuf, char *header)
 			int index = 0;
 			int lines_used = 0;
 
-			inverse_on(mywin1);
-			mvwprintw(mywin1 -> win, nlines + 1, 1, "%s - %d buffered lines", header, pbuf -> curpos);
-			inverse_off(mywin1);
+			ui_inverse_on(mywin1);
+			mvwprintw(mywin1 -> win, nlines + 1, 1, "%s - %d buffered lines", shorten_filename(header, max(24, ncols - 24)), pbuf -> curpos);
+			ui_inverse_off(mywin1);
 
-			if (!no_linewrap) inverse_on(mywin1);
+			if (!no_linewrap) ui_inverse_on(mywin1);
 			mvwprintw(mywin1 -> win, nlines + 1, ncols - 8, "LINEWRAP");
-			if (!no_linewrap) inverse_off(mywin1);
+			if (!no_linewrap) ui_inverse_off(mywin1);
 
 			werase(mywin2 -> win);
 
-			for(;(offset + index) < pbuf -> curpos;)
+			if (!no_linewrap && line_offset > 0)
 			{
-				int prev_lines_used = lines_used;
+				int temp_line_offset = line_offset;
+				int n_chars_to_display_left = strlen((pbuf -> be)[offset].Bline) - temp_line_offset;
+
+				while(lines_used < nlines && n_chars_to_display_left > 0)
+				{
+					scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, pbuf, offset, lines_used, temp_line_offset, 1, show_winnr);
+
+					temp_line_offset += ncols;
+					n_chars_to_display_left -= ncols;
 
-				if (no_linewrap)
 					lines_used++;
+				}
+
+				index++;
+			}
+
+			for(;(offset + index) < pbuf -> curpos && lines_used < nlines;)
+			{
+				int lines_needed = get_lines_needed((pbuf -> be)[offset + index].Bline, ncols);
+
+				if (no_linewrap || lines_needed == 1)
+				{
+					scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, pbuf, offset + index, lines_used, no_linewrap?line_offset:0, no_linewrap, show_winnr);
+					lines_used++;
+				}
 				else
-					lines_used += get_lines_needed((pbuf -> be)[offset + index].Bline, ncols);
-				if (lines_used > nlines)
-					break;
+				{
+					int cur_line_offset = 0;
+
+					while(lines_used < nlines && lines_needed > 0)
+					{
+						scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, pbuf, offset + index, lines_used, cur_line_offset, 1, show_winnr);
+						cur_line_offset += ncols;
+						lines_used++;
+						lines_needed--;
+					}
+				}
 
-				scrollback_displayline(mywin2, pbuf, offset + index, prev_lines_used, line_offset, no_linewrap);
 				index++;
 			}
 
@@ -252,18 +304,32 @@ void scrollback_do(buffer *pbuf, char *header)
 		c = wait_for_keypress(HELP_SCROLLBACK_HELP, 0, NULL, 1);
 		uc = toupper(c);
 
-		if (uc == 'Q' || uc == 'X' || c == 7 || c == KEY_CLOSE || c == KEY_EXIT)
+		if (c == 'q' || uc == 'X' || c == abort_key || c == KEY_CLOSE || c == KEY_EXIT)
 		{
 			break;
 		}
+		else if (c == 'Q' || c == -1)		/* Q: close whole stack of scrollbackwindows, -1: something got closed */
+		{
+			rc = -1;
+			break;
+		}
+		else if (c == 20 && winnrs != NULL)	/* ^t */
+		{
+			show_winnr = 1 - show_winnr;
+			redraw = 2;
+		}
 		else if (c == 'Y')
 		{
 			no_linewrap = !no_linewrap;
 			redraw = 2;
 			line_offset = 0;
 		}
+		else if (c == 't')
+		{
+			statistics_menu();
+		}
 		else if ((c == KEY_LEFT ||
-			  c == KEY_BACKSPACE)
+					c == KEY_BACKSPACE)
 				&& no_linewrap)
 		{
 			if (line_offset > 0)
@@ -314,25 +380,27 @@ void scrollback_do(buffer *pbuf, char *header)
 					c == 'k' ||
 					/* c == 11  || *//* ^k */
 					c == 16)    /* ^p */
-				&& offset > 0)
+				&& (offset > 0 || (!no_linewrap && line_offset > 0)))
 		{
-			int n_to_move = 0;
-
-			wmove(mywin2 -> win, 0, 0);
-
-			do
+			if (no_linewrap)
+			{
+				offset--;
+			}
+			else if (line_offset > 0)
+			{
+				line_offset = max(0, line_offset - ncols);
+			}
+			else
 			{
 				offset--;
 
-				if (no_linewrap)
-					n_to_move = 1;
-				else
-					n_to_move = get_lines_needed((pbuf -> be)[offset].Bline, ncols);
-			} while(n_to_move == 0 && offset > 0);
+				line_offset = (get_lines_needed((pbuf -> be)[offset].Bline, ncols) - 1) * ncols;
+			}
 
-			winsdelln(mywin2 -> win, n_to_move);
+			wmove(mywin2 -> win, 0, 0);
+			winsdelln(mywin2 -> win, 1);
 
-			scrollback_displayline(mywin2, pbuf, offset, 0, line_offset, no_linewrap);
+			scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, pbuf, offset, 0, line_offset, no_linewrap, show_winnr);
 
 			redraw = 1;
 		}
@@ -345,12 +413,23 @@ void scrollback_do(buffer *pbuf, char *header)
 					c == KEY_ENTER)
 				&& offset < (pbuf -> curpos - 1))
 		{
-			int n_to_move = 0;
-
-			while(n_to_move == 0 && offset < (pbuf -> curpos - 1))
+			if (no_linewrap)
 			{
 				offset++;
-				n_to_move = get_lines_needed((pbuf -> be)[offset].Bline, ncols);
+			}
+			else if (strlen((pbuf -> be)[offset].Bline) > (line_offset + ncols))
+			{
+				line_offset += ncols;
+			}
+			else if (offset < (pbuf -> curpos - 1))
+			{
+				if (strlen((pbuf -> be)[offset].Bline) > (line_offset + ncols))
+					line_offset += ncols;
+				else
+				{
+					line_offset = 0;
+					offset++;
+				}
 			}
 
 			redraw = 2;
@@ -365,18 +444,43 @@ void scrollback_do(buffer *pbuf, char *header)
 					c == ('U' - 65 + 1))   /* ^u */
 				&& offset < (pbuf -> curpos - 1))
 		{
-			int dummy = 0;
-			int page_size = (c == 'u' || c == ('U' - 65 + 1)) ? (nlines / 2) : nlines;
-			while(offset < (pbuf -> curpos - 1))
+			if (no_linewrap)
 			{
-				offset++;
-				dummy += get_lines_needed((pbuf -> be)[offset].Bline, ncols);
-				if (dummy > page_size && c == KEY_NPAGE)
-				{
-					offset--;
-					break;
-				}
+				offset += nlines;
+				if (offset >= pbuf -> curpos)
+					offset = pbuf -> curpos - 1;
 			}
+			else
+			{
+				int n_lines_to_move = nlines;
+
+				while(n_lines_to_move > 0 && offset < (pbuf -> curpos))
+				{
+					if (line_offset > 0)
+					{
+						if (line_offset + ncols >= strlen((pbuf -> be)[offset].Bline))
+						{
+							line_offset = 0;
+							offset++;
+							n_lines_to_move--;
+						}
+						else
+						{
+							line_offset += ncols;
+							n_lines_to_move--;
+						}
+					}
+					else
+					{
+						n_lines_to_move -= get_lines_needed((pbuf -> be)[offset].Bline, ncols);
+						offset++;
+					}
+				}
+
+				if (n_lines_to_move < 0)
+					line_offset = (-n_lines_to_move) * ncols;
+			}
+
 			redraw = 2;
 		}
 		else if ((c == KEY_PPAGE ||
@@ -387,35 +491,51 @@ void scrollback_do(buffer *pbuf, char *header)
 					c == 4)         /* ^d */
 				&& offset > 0)
 		{
-			int dummy = 0;
-			int page_size = (c == 'd' || c == 4) ? (nlines / 2) : nlines;
-			while(offset > 0)
+			if (no_linewrap)
 			{
-				offset--;
-				dummy += get_lines_needed((pbuf -> be)[offset].Bline, ncols);
-				if (dummy > page_size)
+				offset -= nlines;
+				if (offset < 0)
+					offset = 0;
+			}
+			else
+			{
+				int n_lines_to_move = nlines;
+
+				if (line_offset)
+					n_lines_to_move -= line_offset / ncols;
+
+				while(n_lines_to_move > 0 && offset > 0)
 				{
-					offset++;
-					break;
+					offset--;
+
+					n_lines_to_move -= get_lines_needed((pbuf -> be)[offset].Bline, ncols);
+
+					if (n_lines_to_move < 0)
+					{
+						line_offset = (get_lines_needed((pbuf -> be)[offset].Bline, ncols) + n_lines_to_move) * ncols;
+					}
 				}
 			}
+					
 			redraw = 2;
 		}
 		else if ((c == KEY_HOME ||
-			  c == 'g' ||
-			  c == '<' ||
-			  c == KEY_SBEG)
+					c == 'g' ||
+					c == '<' ||
+					c == KEY_SBEG)
 				&& offset > 0)
 		{
 			line_offset = offset = 0;
+			redraw = 2;
 		}
 		else if ((c == KEY_END ||
-			  c == 'G' ||
-			  c == '>' ||
-			  c == KEY_SEND)
+					c == 'G' ||
+					c == '>' ||
+					c == KEY_SEND)
 				&& offset < (pbuf -> curpos - 1))
 		{
 			offset = pbuf -> curpos - 1;
+			redraw = 2;
 		}
 		else if (uc == 'R' || c == ('R' - 65 + 1) || c == ('L' - 65 + 1) || c == KEY_REFRESH)
 		{
@@ -423,7 +543,7 @@ void scrollback_do(buffer *pbuf, char *header)
 		}
 		else if (c == ('K' - 65 + 1) || c == KEY_MARK)
 		{
-			scrollback_find_popup(&find);
+			scrollback_find_popup(&find, &case_insensitive);
 
 			if (find)
 			{
@@ -433,7 +553,7 @@ void scrollback_do(buffer *pbuf, char *header)
 				myfree(global_highlight_str);
 				global_highlight_str = NULL;
 
-				if ((rc = regcomp(&global_highlight_re, find, REG_EXTENDED)))
+				if ((rc = regcomp(&global_highlight_re, find, (case_insensitive == MY_TRUE?REG_ICASE:0) | REG_EXTENDED)))
 				{
 					regexp_error_popup(rc, &global_highlight_re);
 					myfree(find);
@@ -451,46 +571,64 @@ void scrollback_do(buffer *pbuf, char *header)
 		{
 			char direction = (c == '?' || c == KEY_SFIND) ? -1 : 1;
 
-			scrollback_find_popup(&find);
+			scrollback_find_popup(&find, &case_insensitive);
 
 			if (find)
 			{
-				int new_f_index;
-
-				redraw = 2; /* force redraw */
-
-				regfree(&global_highlight_re);
-				myfree(global_highlight_str);
-				global_highlight_str = NULL;
-
-				new_f_index = find_string(pbuf, find, 0, direction);
-				if (new_f_index == -1)
+				if (scrollback_search_new_window)
 				{
-					wrong_key();
+					if (scrollback_search_to_new_window(pbuf, header, find, case_insensitive) == -1)
+					{
+						/* cascaded close */
+						rc = -1;
+						break;
+					}
 				}
 				else
 				{
-					offset = new_f_index;
+					int new_f_index;
+
+					redraw = 2; /* force redraw */
+
+					regfree(&global_highlight_re);
+					myfree(global_highlight_str);
+					global_highlight_str = NULL;
+
+					new_f_index = find_string(pbuf, find, 0, direction, case_insensitive);
+					if (new_f_index == -1)
+					{
+						wrong_key();
+					}
+					else
+					{
+						offset = new_f_index;
+						line_offset = 0;
+					}
 				}
 			}
 		}
-		else if (uc == 'n' || c == KEY_NEXT || c == KEY_PREVIOUS || c == KEY_SNEXT)
+		else if (uc == 'N' || c == KEY_NEXT || c == KEY_PREVIOUS || c == KEY_SNEXT)
 		{
 			if (find != NULL)
 			{
 				char direction = (c == 'n' || c == KEY_NEXT) ? 1 : -1;
 				int start_offset = offset + direction;
-				int new_f_index = find_string(pbuf, find, start_offset, direction);
+				int new_f_index = find_string(pbuf, find, start_offset, direction, case_insensitive);
 				if (new_f_index == -1)
+				{
 					wrong_key();
+				}
 				else
 				{
 					redraw = 2; /* force redraw */
 					offset = new_f_index;
+					line_offset = 0;
 				}
 			}
 			else
+			{
 				wrong_key();
+			}
 		}
 		else if (c == 's' || c == KEY_SAVE)
 		{
@@ -514,6 +652,11 @@ void scrollback_do(buffer *pbuf, char *header)
 		{
 			statistics_menu();
 		}
+		else if (c == 20)
+		{
+			toggle_subwindow_nr();
+			redraw = 2;	/* force redraw */
+		}
 		else
 		{
 			wrong_key();
@@ -524,6 +667,8 @@ void scrollback_do(buffer *pbuf, char *header)
 	delete_popup(mywin1);
 
 	myfree(find);
+
+	return rc;
 }
 
 void scrollback(void)
@@ -544,30 +689,31 @@ void scrollback(void)
 	if (window != -1 && lb[window].bufferwhat != 0)
 	{
 		int header_size = strlen(pi[window].filename) + 4;
-		char *header = (char *)mymalloc(header_size + 1, "window header");
+		char *header = (char *)mymalloc(header_size + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		snprintf(header, header_size, "%02d] %s", window, pi[window].filename);
-		scrollback_do(&lb[window], header);
+		scrollback_do(window, &lb[window], NULL, header);
 		myfree(header);
 	}
 }
 
-void merged_scrollback_with_search(char *search_for)
+void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 {
 	int lc_size = nfd * sizeof(int);
-	int *last_check = (int *)mymalloc(lc_size, "last check index");
-	buffer *pbuf = (buffer *)mymalloc(sizeof(buffer), "storage buffer"); 
+	int *last_check = (int *)mymalloc(lc_size, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+	int *winnr = NULL;
+	buffer buf;
 	regex_t reg;
 	int rc;
 
 	memset(last_check, 0x00, lc_size);
-	memset(pbuf, 0x00, sizeof(*pbuf));
+	memset(&buf, 0x00, sizeof(buf));
 
 	/* compile the search string which is supposed to be a valid regular
 	 * expression
 	 */
 	if (search_for)
 	{
-		if ((rc=regcomp(&reg, search_for, REG_EXTENDED | re_case_insensitive?REG_ICASE:0)))
+		if ((rc=regcomp(&reg, search_for, REG_EXTENDED | (case_insensitive == MY_TRUE?REG_ICASE:0))))
 		{
 			regexp_error_popup(rc, &reg);
 			return;
@@ -584,25 +730,21 @@ void merged_scrollback_with_search(char *search_for)
 		char *string;
 		int checked_all = 0;
 
-		LOG("chosen_ts: %f\n", chosen_ts);
-
 		for(loop=0; loop<nfd; loop++)
 		{
 			if (lb[loop].curpos == last_check[loop])
 			{
-				LOG("curpos is last_check\n");
 				checked_all++;
 				continue;
 			}
 
-			if (search_for != NULL)
+			if (search_for != NULL && lb[loop].be[last_check[loop]].Bline != NULL)
 			{
 				rc = regexec(&reg, lb[loop].be[last_check[loop]].Bline, 0, NULL, 0);
 
 				/* did not match? don't add and continue */
 				if (rc == REG_NOMATCH)
 				{
-					LOG("reg nomatch %d / %d: %s\n", loop, last_check[loop], lb[loop].be[last_check[loop]].Bline);
 					continue;
 				}
 
@@ -614,10 +756,8 @@ void merged_scrollback_with_search(char *search_for)
 				}
 			}
 
-			LOG("string found: %s\n", lb[loop].be[last_check[loop]].Bline);
 			if (lb[loop].be[last_check[loop]].ts <= chosen_ts)
 			{
-				LOG("~~~ early\n");
 				chosen_ts = lb[loop].be[last_check[loop]].ts;
 				chosen_win = loop;
 			}
@@ -625,8 +765,6 @@ void merged_scrollback_with_search(char *search_for)
 
 		if (chosen_win == -1)
 		{
-			LOG("not found\n");
-
 			if (checked_all == nfd)
 				break;
 
@@ -639,27 +777,29 @@ void merged_scrollback_with_search(char *search_for)
 			continue;
 		}
 
-		LOG("add!");
-
 		if (!IS_MARKERLINE(lb[chosen_win].be[last_check[chosen_win]].pi))
 		{
 			/*** ADD LINE TO BUFFER ***/
-			pbuf -> be = (buffered_entry *)myrealloc(pbuf -> be, sizeof(buffered_entry) * (pbuf -> curpos + 1), "merged_scrollback_with_search: buffered_entry");
-			curline = pbuf -> curpos++;
+			buf.be = (buffered_entry *)myrealloc(buf.be, sizeof(buffered_entry) * (buf.curpos + 1), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+			winnr  = (int *)myrealloc(winnr, sizeof(int) * (buf.curpos + 1), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+			curline = buf.curpos++;
 			/* add the logline itself */
 			string = lb[chosen_win].be[last_check[chosen_win]].Bline;
 			if (string)
-				(pbuf -> be)[curline].Bline = mystrdup(string, "merged_scrollback_with_search: print_and_buffer");
+				buf.be[curline].Bline = mystrdup(string, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			else
-				(pbuf -> be)[curline].Bline = NULL;
+				buf.be[curline].Bline = NULL;
 			/* remember pointer to subwindow (required for setting colors etc.) */
-			(pbuf -> be)[curline].pi = lb[chosen_win].be[last_check[chosen_win]].pi;
+			buf.be[curline].pi = lb[chosen_win].be[last_check[chosen_win]].pi;
+			buf.be[curline].ts = lb[chosen_win].be[last_check[chosen_win]].ts;
+			/* remember window nr. */
+			winnr[curline] = chosen_win;
 		}
 
 		last_check[chosen_win]++;
 	}
 
-	if (pbuf -> curpos == 0)
+	if (buf.curpos == 0)
 		error_popup("Search in all windows", -1, "Nothing found.");
 	else
 	{
@@ -669,20 +809,68 @@ void merged_scrollback_with_search(char *search_for)
 		{
 			char *help = "Searched for: ";
 			int len = strlen(help) + strlen(search_for) + 1;
-			header = mymalloc(len, "merged_scrollback_with_search header (1)");
+			header = mymalloc(len, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			snprintf(header, len, "%s%s", help, search_for);
 		}
 		else
 		{
 			char *help = "Merge view";
-			header = mymalloc(strlen(help) + 1, "merged_scrollback_with_search header (2)");
+			header = mymalloc(strlen(help) + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			sprintf(header, "%s", help);
 		}
 
-		scrollback_do(pbuf, header);
+		scrollback_do(-1, &buf, winnr, header);
 
 		myfree(header);
 	}
 
-	delete_be_in_buffer(pbuf);
+	delete_be_in_buffer(&buf);
+
+	myfree(winnr);
+
+	myfree(last_check);
+}
+
+int scrollback_search_to_new_window(buffer *pbuf, char *org_title, char *find_str, mybool_t case_insensitive)
+{
+	int loop, rc;
+	regex_t regex;
+	buffer cur_lb;
+	char *new_header;
+
+	/* compile the searchstring (which can be a regular expression) */
+	if ((rc = regcomp(&regex, find_str, REG_EXTENDED | (case_insensitive == MY_TRUE?REG_ICASE:0))))
+	{
+		regexp_error_popup(rc, &regex);
+		return 0;
+	}
+
+	memset(&cur_lb, 0x00, sizeof(buffer));
+
+	for(loop=0; loop<pbuf -> curpos; loop++)
+	{
+		if ((pbuf -> be)[loop].Bline == NULL)
+			continue;
+
+		if (regexec(&regex, (pbuf -> be)[loop].Bline, 0, NULL, 0) == 0)
+		{
+			cur_lb.be = myrealloc(cur_lb.be, (cur_lb.curpos + 1) * sizeof(buffered_entry), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+			cur_lb.be[cur_lb.curpos].Bline = (pbuf -> be)[loop].Bline;
+			cur_lb.be[cur_lb.curpos].pi    = (pbuf -> be)[loop].pi;
+			cur_lb.curpos++;
+		}
+	}
+
+	new_header = (char *)mymalloc(strlen(org_title) + 1 + strlen(find_str) + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+	sprintf(new_header, "%s %s", org_title, find_str);
+
+	rc = scrollback_do(-1, &cur_lb, NULL, new_header);
+
+	myfree(new_header);
+
+	myfree(cur_lb.be);
+
+	regfree(&regex);
+
+	return rc;
 }
