@@ -2,8 +2,8 @@ int swap_window(void);
 int delete_window(void);
 int hide_window(void);
 char ask_negate_regexp(NEWWIN *win, int line);
-int select_colorschemes(int **color_schemes, int *n_color_schemes);
-char ask_colors(NEWWIN *win, int line, char cur, char *fieldnr, char **fielddel, int **color_schemes, int *n_color_schemes, myattr_t *attrs);
+int select_schemes(void *schemes_in, int n_schemes_in, scheme_t tscheme, int_array_t *schemes_out);
+char ask_colors(NEWWIN *win, int line, char cur, char *fieldnr, char **fielddel, int_array_t *color_schemes, myattr_t *attrs);
 char ask_regex_type(NEWWIN *win, int line);
 int add_window(void);
 int toggle_colors(void);
@@ -38,3 +38,10 @@ void search_in_all_windows(void);
 void highlight_in_all_windows(void);
 void toggle_regexp_case_insensitive(void);
 void restart_window(void);
+void inverse_on(NEWWIN *win);
+void inverse_off(NEWWIN *win);
+void select_conversionschemes(void);
+void toggle_subwindow_nr(void);
+void clear_a_buffer(void);
+void clear_all_buffers(void);
+void ask_case_insensitive(mybool_t *pcase_insensitive);
