@@ -1,4 +1,5 @@
 #define _LARGEFILE64_SOURCE	/* required for GLIBC to enable stat64 and friends */
+#include "doassert.h"
 #include <errno.h>
 #include <signal.h>
 #include <stdlib.h>
@@ -22,6 +23,9 @@
 #endif
 #include <dirent.h>
 #include <ctype.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <fcntl.h>
 
 #include "version.h"
 #include "error.h"
@@ -32,6 +36,19 @@
 #include "utils.h"
 
 
+#ifdef _DEBUG
+void check_fd(int fd, char *file, const char *function, int line)
+{
+	if (fcntl(fd, F_GETFL) == -1)
+	{
+		if (errno == EBADF)
+			error_exit(file, function, line, "Not a valid filedescriptor.\n");
+	}
+}
+#else
+#define check_fd(x, y, z, a)
+#endif
+
 int find_path_max(void)
 {
 #ifdef PATH_MAX
@@ -40,7 +57,7 @@ int find_path_max(void)
 	int path_max = pathconf("/", _PC_PATH_MAX);
 	if (path_max <= 0)
 	{
-		if (errno) error_exit("find_path_max: pathconf(_PC_PATH_MAX) failed\n");
+		if (errno) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "pathconf() failed\n");
 
 		path_max = 4096;
 	}
@@ -71,7 +88,7 @@ ssize_t WRITE(int fd, char *whereto, size_t len, char *for_whom)
 		if (rc == -1)
 		{
 			if (errno != EINTR && errno != EAGAIN)
-				error_exit("WRITE: Problem writing to filedescriptor while processing %s\n", for_whom);
+				error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Problem writing to file descriptor while processing %s.\n", for_whom);
 		}
 		else if (rc == 0)
 		{
@@ -103,7 +120,7 @@ void get_load_values(double *v1, double *v2, double *v3)
 		 * calls start to fail, something must be
 		 * really wrong
 		 */
-		error_exit("get_load: sysinfo() failed\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "sysinfo() failed\n");
 	}
 
 	*v1 = (double)si.loads[0] / scale;
@@ -114,7 +131,7 @@ void get_load_values(double *v1, double *v2, double *v3)
 	if (getloadavg(loadavg, 3) == -1)
 	{
 		/* see comment on sysinfo() */
-		error_exit("get_load: getloadavg() failed\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "getloadavg() failed\n");
 	}
 	*v1 = loadavg[0];
 	*v2 = loadavg[1];
@@ -131,14 +148,16 @@ int get_vmsize(pid_t pid)
 #if defined(linux)
 	FILE *fh;
 	int path_max = find_path_max();
-	char *path = mymalloc(path_max, "path");
+	char *path = mymalloc(path_max, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+
+	assert(pid > 1);
 
 	snprintf(path, path_max, "/proc/%d/stat", pid);
 
 	fh = fopen(path, "r");
 	if (fh)
 	{
-		char *dummystr = mymalloc(path_max, "get_vmsize: temp string");
+		char *dummystr = mymalloc(path_max, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		char dummychar;
 		int dummy;
 
@@ -156,6 +175,15 @@ int get_vmsize(pid_t pid)
 	return vmsize;
 }
 
+int mykillpg(pid_t pid, int sigtype)
+{
+#ifdef AIX
+	return kill(pid, sigtype);
+#else
+	return killpg(pid, sigtype);
+#endif
+}
+
 /** stop_process
  * - in:      int pid  pid of process
  * - returns: nothing
@@ -164,42 +192,41 @@ int get_vmsize(pid_t pid)
  * is send so the process gets the possibility to gracefully exit. if it doesn't do that
  * in 100 microseconds, it is terminated
  */
-void stop_process(int pid)
+void stop_process(pid_t pid)
 {
-	if (pid < 2)
-		return;
+	assert(pid > 1);
 
-	if (killpg(pid, SIGTERM) == -1)
+	if (mykillpg(pid, SIGTERM) == -1)
 	{
 		if (errno != ESRCH)
-			error_exit("stop_process: problem stopping child-process (%d)!\n", pid);
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Problem stopping child process with PID %d (SIGTERM).\n", pid);
 	}
 
 	usleep(1000);
 
 	/* process still exists? */
-	if (killpg(pid, SIGTERM) == 0)
+	if (mykillpg(pid, SIGTERM) == 0)
 	{
 		/* sleep for a millisecond... */
 		usleep(1000);
 
 		/* ...and then really terminate the process */
-		if (killpg(pid, SIGKILL) == -1)
+		if (mykillpg(pid, SIGKILL) == -1)
 		{
 			if (errno != ESRCH)
-				error_exit("stop_process: problem killing child-process (%d)!\n", pid);
+				error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Problem stopping child process with PID %d (SIGKILL).\n", pid);
 		}
 	}
 	else if (errno != ESRCH)
-		error_exit("stop_process: problem stopping child-process (%d)!\n", pid);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Problem stopping child process with PID %d (SIGTERM).\n", pid);
 
-	/* wait for the last remainder of the exited process to go away,
+	/* wait for the last remainder of the died process to go away,
 	 * otherwhise we'll find zombies on our way
 	 */
 	if (waitpid(pid, NULL, WNOHANG | WUNTRACED) == -1)
 	{
 		if (errno != ECHILD)
-			error_exit("stop_process: waitpid failed\n");
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "waitpid() failed\n");
 	}
 }
 
@@ -214,6 +241,8 @@ void delete_array(char **list, int n)
 {
 	int loop;
 
+	assert(n >= 0);
+
 	for(loop=n-1; loop>=0; loop--)
 		myfree(list[loop]);
 
@@ -222,7 +251,11 @@ void delete_array(char **list, int n)
 
 int find_char_offset(char *str, char what)
 {
-	char *pnt = strchr(str, what);
+	char *pnt;
+
+	assert(what > 0);
+
+	pnt = strchr(str, what);
 	if (!pnt)
 		return -1;
 
@@ -236,7 +269,7 @@ int file_info(char *filename, off64_t *file_size, time_field_t tft, time_t *ts,
 	if (stat64(filename, &buf) == -1)
 	{
 		if (errno != ENOENT)
-			error_exit("file_info: error while doing stat() on file %s\n", filename);
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while obtaining details of file %s.\n", filename);
 
 		return -1;
 	}
@@ -251,8 +284,10 @@ int file_info(char *filename, off64_t *file_size, time_field_t tft, time_t *ts,
 			*ts = buf.st_mtime;
 		else if (tft == TT_CTIME)
 			*ts = buf.st_ctime;
-		else if (tft != 0)
-			error_exit("file_info: invalid timefield (%d)\n", (int)tft);
+		else
+		{
+			assert(tft == 0);
+		}
 	}
 
 	if (mode) *mode = buf.st_mode;
@@ -266,7 +301,7 @@ int file_exist(char *filename)
 	int rc = stat(filename, &buf);
 
 	if (rc == -1 && errno != ENOENT)
-		error_exit("file_exist(%s) failed\n", filename);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "stat() on file %s failed.\n", filename);
 
 	return rc;
 }
@@ -279,9 +314,11 @@ char * convert_regexp_error(int error, const regex_t *preg)
 	char *error_out = NULL;
 	const int max_err_len = 256;
 
-	if (error != 0 && error != REG_NOMATCH)
+	assert(error != 0);
+
+	if (error != REG_NOMATCH)
 	{
-		error_out = (char *)mymalloc(max_err_len + len + 1, "convert_regexp_error: regexp error string");
+		error_out = (char *)mymalloc(max_err_len + len + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 		memcpy(error_out, multitail_string, len);
 
@@ -292,23 +329,11 @@ char * convert_regexp_error(int error, const regex_t *preg)
 	return error_out;
 }
 
-/* I'm sorry for this: */
-char check_date(void)
-{
-	time_t now = time(NULL);
-	struct tm *ptm = localtime(&now);
-
-	if (ptm -> tm_mon == (4 - 1) && ptm -> tm_mday == 2) /* April the 2nd? */
-	{
-		return 1;
-	}
-
-	return 0;
-}
-
 char * amount_to_str(long long int amount)
 {
-	char *out = mymalloc(AMOUNT_STR_LEN, "amount_to_str: converted amount string");	/* ...XB\0 */
+	char *out = mymalloc(AMOUNT_STR_LEN, __FILE__, __PRETTY_FUNCTION__, __LINE__);	/* ...XB\0 */
+
+	assert(amount >= 0);
 
 	if (amount >= M_GB)	/* GB */
 		snprintf(out, AMOUNT_STR_LEN, "%dGB", (int)((amount + M_GB - 1) / M_GB));
@@ -326,7 +351,7 @@ struct passwd *getuserinfo(void)
 {
 	struct passwd *pp = getpwuid(geteuid());
 	if (!pp)
-		error_exit("getuserinfo: failed to get passwdstructure for effective user id %d\n", geteuid());
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Failed to get passwd-structure for effective user id %d.\n", geteuid());
 
 	return pp;
 }
@@ -368,6 +393,8 @@ int myopen(char *path, int mode)
 
 int myclose(int fd)
 {
+	check_fd(fd, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+
 	for(;;)
 	{
 		if (close(fd) == -1)
@@ -388,6 +415,8 @@ char * shorten_filename(char *in, int max_len)
 	int len = strlen(in);
 	int cutlen, dummy;
 
+	assert(max_len >= 0);
+
 	if (len <= max_len)
 		return in;
 
@@ -408,7 +437,7 @@ double get_ts(void)
 	struct timezone tz;
 
 	if (gettimeofday(&ts, &tz) == -1)
-		error_exit("get_ts: gettimeofday failed");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "gettimeofday() failed");
 
 	return (((double)ts.tv_sec) + ((double)ts.tv_usec)/1000000.0);
 }
@@ -417,7 +446,7 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 {
 	DIR *dir;
 	struct dirent *entry;
-	char *cur_dir = mymalloc(find_path_max() + 1, "match_files: current file");
+	char *cur_dir = mymalloc(find_path_max() + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	char *fname;
 	char **list = NULL;
 	int nfound = 0;
@@ -427,14 +456,14 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 	char *slash = strrchr(search_for, '/');
 	if (slash)
 	{
-		fname = mystrdup(slash + 1, "match_files: filename");
+		fname = mystrdup(slash + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		*(slash + 1) = 0x00;
-		*path = mystrdup(search_for, "match_files: current path");
+		*path = mystrdup(search_for, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	}
 	else
 	{
-		*path = mystrdup("./", "match_files: current path");
-		fname = mystrdup(search_for, "match_files: filename");
+		*path = mystrdup("./", __FILE__, __PRETTY_FUNCTION__, __LINE__);
+		fname = mystrdup(search_for, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	}
 	fname_size = strlen(fname);
 	path_len = strlen(*path);
@@ -456,16 +485,16 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 			struct stat finfo;
 
 			/* get filename */
-			list = (char **)myrealloc(list, (nfound + 1) * sizeof(char *), "match_files: directory list");
-			list[nfound] = mystrdup(entry -> d_name, "match_files dir entry");
+			list = (char **)myrealloc(list, (nfound + 1) * sizeof(char *), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+			list[nfound] = mystrdup(entry -> d_name, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 			/* check if the file is a directory */
-			*isdir = (char *)myrealloc(*isdir, (nfound + 1) * sizeof(char), "ismatch_files: -dir list");
+			*isdir = (char *)myrealloc(*isdir, (nfound + 1) * sizeof(char), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			strncpy(&cur_dir[path_len], entry -> d_name, max(0,find_path_max() - path_len));
 			if (stat(cur_dir, &finfo) == -1)
 			{
 				if (errno != ENOENT)	/* file did not disappear? then something is very wrong */
-					error_exit("match_files: stat error on %s\n", cur_dir);
+					error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error while invoking stat() %s.\n", cur_dir);
 			}
 			(*isdir)[nfound] = S_ISDIR(finfo.st_mode)?1:0;
 
@@ -474,7 +503,7 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 	}
 
 	if (closedir(dir) == -1)
-		error_exit("match_files: closedir failed\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "closedir() failed\n");
 
 	/*	qsort( (void *)list, (size_t)nfound, sizeof(char *), compare_filenames); */
 	for(s1=0; s1<(nfound - 1); s1++)
@@ -504,14 +533,16 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 void setup_for_childproc(int fd, char close_fd_0, char *term)
 {
 	int term_len = strlen(term) + 5;
-	char *dummy = (char *)mymalloc(term_len + 1, "setup_for_childproc: TERM env var");
+	char *dummy = (char *)mymalloc(term_len + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
-	if (close_fd_0) if (-1 == myclose(0)) error_exit("setup_for_childproc: close failed\n");
-	if (-1 == myclose(1)) error_exit("setup_for_childproc: close failed\n");
-	if (-1 == myclose(2)) error_exit("setup_for_childproc: close failed\n");
-	if (close_fd_0) if (-1 == mydup(fd)) error_exit("setup_for_childproc: dup failed\n");
-	if (-1 == mydup(fd)) error_exit("setup_for_childproc: dup failed\n");
-	if (-1 == mydup(fd)) error_exit("setup_for_childproc: dup failed\n");
+	check_fd(fd, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+
+	if (close_fd_0) if (-1 == myclose(0)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "close() failed\n");
+	if (-1 == myclose(1)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "close() failed\n");
+	if (-1 == myclose(2)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "close() failed\n");
+	if (close_fd_0) if (-1 == mydup(fd)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "dup() failed\n");
+	if (-1 == mydup(fd)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "dup() failed\n");
+	if (-1 == mydup(fd)) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "dup() failed\n");
 
 	/*
 	 * not doing this: it also clears the 'PATH'
@@ -527,6 +558,7 @@ void setup_for_childproc(int fd, char close_fd_0, char *term)
 	 */
 
 	/* set terminal */
+	assert(term != NULL);
 	snprintf(dummy, term_len, "TERM=%s", term);
 	if (putenv(dummy) == -1)
 		fprintf(stderr, "setup_for_childproc: Could not set TERM environment-variable (%s): %s (%d)\n", dummy, strerror(errno), errno);
@@ -538,7 +570,7 @@ int open_null(void)
 {
 	int fd = myopen("/dev/null", O_RDWR);
 	if (fd == -1)
-		error_exit("open_null: Failed to open /dev/null\n");
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Failed to open file /dev/null.\n");
 
 	return fd;
 }
@@ -562,14 +594,11 @@ char * find_most_recent_file(char *filespec, char *cur_file)
 	unsigned int loop;
 	time_t prev_ts = (time_t)0;
 
-	LOG("find file for %s\n", filespec);
-
 	/* get timestamp of previous file */
 	if (cur_file)
 	{
 		if (file_info(cur_file, NULL, TT_MTIME, &prev_ts, NULL) == -1)
 		{
-			LOG("could not get fileinfo for %s\n", cur_file);
 			prev_ts = (time_t)0;
 		}
 	}
@@ -578,14 +607,11 @@ char * find_most_recent_file(char *filespec, char *cur_file)
 #if defined(__APPLE__) || defined(__CYGWIN__)
 	if (glob(filespec, GLOB_ERR | GLOB_NOSORT, NULL, &files) != 0)
 #else
-		if (glob(filespec, GLOB_ERR | GLOB_NOSORT | GLOB_NOESCAPE, NULL, &files) != 0)
+	if (glob(filespec, GLOB_ERR | GLOB_NOSORT | GLOB_NOESCAPE, NULL, &files) != 0)
 #endif
-		{
-			LOG("glob failed %d\n", errno);
-			return NULL;
-		}
-
-	LOG("number of files found: %d\n", files.gl_pathc);
+	{
+		return NULL;
+	}
 
 	/* see if any of them is more recent than the current one */
 	for(loop=0; loop<files.gl_pathc; loop++)
@@ -599,7 +625,6 @@ char * find_most_recent_file(char *filespec, char *cur_file)
 		/* get the modificationtime of this found file */
 		if (file_info(files.gl_pathv[loop], NULL, TT_MTIME, &new_ts, NULL) == -1)
 		{
-			LOG("loop: failed fetching info on file %s: %d\n", files.gl_pathv[loop], errno);
 			new_ts = (time_t)0;
 		}
 
@@ -614,7 +639,7 @@ char * find_most_recent_file(char *filespec, char *cur_file)
 	/* found a file? then remember the filename */
 	if (selected_file != NULL)
 	{
-		selected_file = mystrdup(selected_file, "find_most_recent_file: new file filename");
+		selected_file = mystrdup(selected_file, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	}
 
 	globfree(&files);
@@ -648,6 +673,8 @@ int mydup(int old_fd)
 {
 	int new_fd = -1;
 
+	check_fd(old_fd, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+
 	for(;;)
 	{
 		new_fd = dup(old_fd);
@@ -657,7 +684,7 @@ int mydup(int old_fd)
 			if (errno == EINTR)
 				continue;
 
-			error_exit("mydup: dup() failed\n");
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "dup() failed\n");
 		}
 
 		break;
@@ -683,43 +710,44 @@ char * term_t_to_string(term_t term)
 	return "dumb";
 }
 
-void get_now_ts(char *format_str, char *dest, int dest_size)
+void double_ts_to_str(dtime_t ts, char *format_str, char *dest, int dest_size)
 {
-	time_t now = time(NULL);
+	time_t now = (time_t)ts;
 	struct tm *ptm = localtime(&now);
-	if (!ptm) error_exit("get_now_ts: localtime() failed\n");
+	if (!ptm) error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "localtime() failed\n");
+
+	assert(ts > 0);
+	assert(dest_size > 0);
 
 	(void)strftime(dest, dest_size, format_str, ptm);
 }
 
-void double_ts_to_str(double ts, char *format_str, char *dest, int dest_size)
+void get_now_ts(char *format_str, char *dest, int dest_size)
 {
-	time_t now = (time_t)ts;
-	struct tm *ptm = localtime(&now);
-	if (!ptm) error_exit("double_ts_to_str: localtime() failed\n");
-
-	(void)strftime(dest, dest_size, format_str, ptm);
+	double_ts_to_str(get_ts(), format_str, dest, dest_size);
 }
 
 void duplicate_re_array(re *pre_in, int n_rein, re **pre_out, int *n_reout)
 {
 	int loop, old_n = *n_reout;
 
+	assert(n_rein >= 0);
+
 	*n_reout += n_rein;
 	if (*n_reout)
 	{
-		*pre_out = (re *)myrealloc(*pre_out, (*n_reout) * sizeof(re), "duplicate re array");
+		*pre_out = (re *)myrealloc(*pre_out, (*n_reout) * sizeof(re), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 		for(loop=0; loop<n_rein; loop++)
 		{
 			memcpy(&(*pre_out)[loop + old_n], &pre_in[loop], sizeof(re));
 
 			memset(&(*pre_out)[loop + old_n].regex, 0x00, sizeof(regex_t));
-			(*pre_out)[loop + old_n].regex_str = mystrdup(pre_in[loop].regex_str, "regex str");
+			(*pre_out)[loop + old_n].regex_str = mystrdup(pre_in[loop].regex_str, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			compile_re(&(*pre_out)[loop + old_n].regex, (*pre_out)[loop + old_n].regex_str);
 
 			if (pre_in[loop].cmd)
-				(*pre_out)[loop + old_n].cmd = mystrdup(pre_in[loop].cmd, "command to invoke when re matches");
+				(*pre_out)[loop + old_n].cmd = mystrdup(pre_in[loop].cmd, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		}
 	}
 	else
@@ -732,22 +760,22 @@ void duplicate_es_array(strip_t *pes_in, int n_esin, strip_t **pes_out, int *n_e
 {
 	int loop, old_n = *n_esout;
 
+	assert(n_esin >= 0);
+
 	*n_esout += n_esin;
-	*pes_out = (strip_t *)myrealloc(*pes_out, (*n_esout) * sizeof(strip_t), "duplicate es array");
+	*pes_out = (strip_t *)myrealloc(*pes_out, (*n_esout) * sizeof(strip_t), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 	for(loop=0; loop<n_esin; loop++)
 	{
 		memcpy(&(*pes_out)[loop + old_n], &pes_in[loop], sizeof(strip_t));
 
-		LOG("dup type: %d\n", (*pes_out)[loop + old_n].type);
-
 		memset(&(*pes_out)[loop + old_n].regex, 0x00, sizeof(regex_t));
-		(*pes_out)[loop + old_n].regex_str = mystrdup(pes_in[loop].regex_str, "regex str");
+		(*pes_out)[loop + old_n].regex_str = mystrdup(pes_in[loop].regex_str, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 		compile_re(&(*pes_out)[loop + old_n].regex, (*pes_out)[loop + old_n].regex_str);
 
 		if (pes_in[loop].del)
-			(*pes_out)[loop + old_n].del = mystrdup(pes_in[loop].del, "delimiter");
+			(*pes_out)[loop + old_n].del = mystrdup(pes_in[loop].del, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	}
 }
 
@@ -782,13 +810,15 @@ void compile_re(regex_t *whereto, char *what)
 	/* compile & store regular expression */
 	int rc = regcomp(whereto, what, REG_EXTENDED);
 	if (rc != 0)
-		error_exit("failed to compile regular expression '%s': %s\n", what, convert_regexp_error(rc, whereto));
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Failed to compile regular expression '%s'.\nReason: %s\n", what, convert_regexp_error(rc, whereto));
 }
 
 void grow_mem_if_needed(char **what, int *cur_len, int requested_len)
 {
 	char changed = 0;
 
+	assert(requested_len > 0);
+
 	while(*cur_len < requested_len)
 	{
 		changed = 1;
@@ -800,11 +830,13 @@ void grow_mem_if_needed(char **what, int *cur_len, int requested_len)
 	}
 
 	if (changed)
-		*what = myrealloc(*what, *cur_len, "grow_mem_if_needed");
+		*what = myrealloc(*what, *cur_len, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 }
 
 int READ(int fd, char *where_to, int max_len, char *for_whom)
 {
+	assert(max_len > 0);
+
 	for(;;)
 	{
 		int rc = read(fd, where_to, max_len);
@@ -812,7 +844,7 @@ int READ(int fd, char *where_to, int max_len, char *for_whom)
 			return rc;
 
 		if (errno != EINTR && errno != EAGAIN)
-			error_exit("do_convert: error writing to script '%s'\n", for_whom);
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Error writing to fd for %s.\n", for_whom);
 	}
 }
 
@@ -823,7 +855,7 @@ int get_value_arg(char *par, char *string, valcheck_t check)
 	int multiplier = 1;
 
 	if (!string)
-		error_exit("%s needs a parameter\n", par);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "%s needs a parameter.\n", par);
 
 	len = strlen(string);
 
@@ -845,34 +877,36 @@ int get_value_arg(char *par, char *string, valcheck_t check)
 				break;
 			}
 			else
-				error_exit("%s needs a value as parameter\n", par);
+				error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "%s needs a value as parameter.\n", par);
 		}
 	}
 
 	result = strtol(string, NULL, 10);
 
 	if (result > INT_MAX || result == LONG_MIN || result == LONG_MAX)
-		error_exit("value %s for parameter %s is too large\n", string, par);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Value %s for parameter %s is too large.\n", string, par);
 
 	result *= multiplier;
 
 	if (check == VAL_ZERO_POSITIVE)
 	{
 		if (result < 0)
-			error_exit("value for %s must be >= 0\n", par);
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Value for %s must be >= 0.\n", par);
 	}
 	else if (check == VAL_POSITIVE_NOT_1)
 	{
 		if (result < 0 || result == 1)
-			error_exit("value for %s must be >= 0 and not 1\n", par);
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Value for %s must be >= 0 and must not be 1.\n", par);
 	}
 	else if (check == VAL_POSITIVE)
 	{
 		if (result <= 0)
-			error_exit("value for %s must be > 0\n", par);
+			error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Value for %s must be > 0.\n", par);
 	}
 	else
-		error_exit("get_value_arg: internal error (%d)\n", check);
+	{
+		assert(0);
+	}
 
 	return (int)result;
 }
@@ -881,8 +915,8 @@ void add_to_iat(int_array_t *piat, int element)
 {
 	if (piat -> n == piat -> size)
 	{
-		piat -> size = (piat -> size?piat -> size:8) * 2;
-		piat -> elements = (int *)myrealloc(piat -> elements, piat -> size * sizeof(int), "int_array_t");
+		piat -> size = USE_IF_SET(piat -> size, 8) * 2;
+		piat -> elements = (int *)myrealloc(piat -> elements, piat -> size * sizeof(int), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	}
 
 	(piat -> elements)[piat -> n] = element;
@@ -909,10 +943,7 @@ void free_iat(int_array_t *piat)
 
 int get_iat_element(int_array_t *piat, int index)
 {
-#ifdef _DEBUG
-	if (index >= piat -> n)
-		error_exit("get_iat_element: index out of bounds (current: %d, # elements: %d)\n", index, piat -> n);
-#endif
+	assert(index < piat -> n);
 
 	return (piat -> elements)[index];
 }
@@ -932,13 +963,13 @@ char *gethome(char *user)
 		{
 			if (strcmp(p -> pw_name, user) == 0)
 			{
-				path = mystrdup(p -> pw_dir, "gethome");
+				path = mystrdup(p -> pw_dir, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 				break;
 			}
 		}
 		else if (p -> pw_uid == getuid())
 		{
-			path = mystrdup(p -> pw_dir, "gethome");
+			path = mystrdup(p -> pw_dir, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			break;
 		}
 	}
@@ -955,7 +986,7 @@ char *myrealpath(char *in)
 	char *pout;
 
 	if (in[0] != '~')
-		return mystrdup(in, "myrealpath");
+		return mystrdup(in, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 	if (in[1] != '/')
 	{
@@ -974,7 +1005,7 @@ char *myrealpath(char *in)
 			pin = in + len + 1;
 		}
 
-		user = (char *)mymalloc(len + 1, "copy of username");
+		user = (char *)mymalloc(len + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		memcpy(user, &in[1], len);
 		user[len] = 0x00;
 
@@ -989,12 +1020,14 @@ char *myrealpath(char *in)
 	}
 
 	if (!home)
-		error_exit("Cannot expand path %s: home directory not found!\n", in);
+		error_exit(__FILE__, __PRETTY_FUNCTION__, __LINE__, "Cannot expand path %s:\nhome directory not found.\n", in);
 
 	home_len = strlen(home);
-	pout = (char *)mymalloc(home_len + strlen(in) + 1, "realpath");
+	pout = (char *)mymalloc(home_len + strlen(in) + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 	sprintf(pout, "%s/%s", home, pin);
 
+	myfree(home);
+
 	return pout;
 }
